__all__ = [
	"DataForSocket",
	"DataFlag",
	"InitTitleNameFlag",
	"EndSend",
	"SOCKET_FILE",
	"ServerError",
	"ViewRoot",
]

from pickle import dumps, loads
from socket import socket
from typing import List, Tuple, Final, Optional

from console_debugger.helpful.template_obj import rel_path

DataFlag: Final[bytes] = b'\0'  # Обычные данные
InitTitleNameFlag: Final[bytes] = b'\1'  # Нужно создать консоли
EndSend: Final[bytes] = b'\2'  # Если данные не удалось распаковать
MyKey: Final[str] = "TRUE_CONNECT"  # Ключ подтверждения того что мы получились на правильный порт
SIZE_BUFFER: Final[int] = 8  # Размер для числа которое указывает размер следующего за ним сегмента (Размер в битах)
SOCKET_FILE: Final[str] = rel_path(-1, "console_debugger.socket")  # Путь к сокет файлу


class ServerError(BaseException):    ...


class DataForSocket:

	# SERVER
	@staticmethod
	def SendTrueConnect(user: socket):
		"""
		Отправить ключевое слово, в подтверждение о подключение
		"""
		user.send(MyKey.encode("ascii"))

	@staticmethod
	def GetDataObj(user: socket) -> Tuple[bytes, int, List[str]]:
		"""
		Получить из сокета, данные сформированные `SendDataObj()`
		:return: флаг, id, данные
		"""
		size_data = user.recv(SIZE_BUFFER)
		if size_data:
			data_bytes = user.recv(int.from_bytes(size_data, byteorder="big"))
			# with open("test.txt", "a")as f:
			# 	print(f"{d}:{len(d)}", file=f)
			# 	print(f"{data}:{len(data)}", file=f)
			# return DataFlag, 0, ["0"]
			return loads(data_bytes)
		else:
			return EndSend, 0, [""]

	# CLIENT
	@staticmethod
	def CheckResponseWithServer(data_validate: bytes) -> bool:
		"""
		Проверить ответ сервера на корректность.
		"""
		return True if data_validate.decode("ascii") == MyKey else False

	@staticmethod
	def SendInitTitleName(client_socket: socket, init_title_name: List[str]):
		"""
		Отправить заголовки консолей
		"""
		data = dumps(
			(InitTitleNameFlag, -1, init_title_name),
			protocol=3)
		len_ = len(data).to_bytes(SIZE_BUFFER, byteorder='big')
		# print("SendInitTitleName")
		# print(f"{data}:{len(data)}")
		# print(f"{len_}: {len(len_)}")
		client_socket.send(len_)
		client_socket.send(data)

	@staticmethod
	def SendDataObj(client_socket: socket, id_: int, text_send: List[str]):
		"""
		Отправить на сервер, данные для конкретной консоли
		"""
		data = dumps(
			(DataFlag, id_, text_send),
			protocol=3)
		len_ = len(data).to_bytes(SIZE_BUFFER, byteorder='big')
		# print("SendDataObj")
		# print(f"{data}:{len(data)}")
		# print(f"{len_}: {len(len_)}")
		client_socket.send(len_)
		client_socket.send(data)


class ViewRoot:

	def __init__(self):
		from console_debugger.logic.mg_get_socket import MgGetSocket
		self.SeverGet: Optional[MgGetSocket] = None

	def PrintInfo(self, text: str):        ...

	def UpdateTitle(self, l_text: List[str]):        ...

	def SendTextInIndex(self, index: int, data: str):        ...



__all__ = ["dopen",
           "dstyle",
           "dDEBUG",
           "dINFO",
           "printD",
           "dWARNING",
           "dEXCEPTION",
           ]

from io import DEFAULT_BUFFER_SIZE
from os.path import dirname
from typing import Union, List, Any


def add_sys_path_if_not(add_path: str, sys_path: List[str]):
	"""
	Добавить путь если его нет
	:param add_path: 
	:param sys_path: 
	:return: 
	"""
	if add_path not in sys_path:
		sys_path.append(add_path)


def rel_path(slice_end: int, add_path: str = ""):
	"""
	Обрезает относительный путь, и может добавлять к нему другой путь:
	:param slice_end: Срез с конца
	:param add_path: Добавить путь
	:return:
	"""
	p = dirname(__file__).replace("\\", "/").split("/")[:slice_end]
	if add_path:
		p.append(add_path.replace("\\", "/"))
	return "/".join(p)


class StyleText:
	def __init__(self, present_text: str, style_text: str):        ...

	def __repr__(self) -> str: ...

	def __str__(self) -> str: ...


class Debugger:
	def __call__(self, textOutput: Union[str, StyleText], *args, sep=' ', end='\n'): ...


def printD(name_instance: Debugger, text: Any, *args, **kwargs):
	name_instance(text, *args, **kwargs)


def dopen(file, mode='a', buffering=None, encoding=None, errors=None, newline=None, closefd=True):
	return {"file": file,
	        "mode": mode,
	        "buffering": buffering if buffering else DEFAULT_BUFFER_SIZE,
	        "encoding": encoding,
	        "errors": errors,
	        "newline": newline,
	        "closefd": closefd}


def dstyle(color: str = None,
           bg_color: str = None,
           attrs: List[str] = None,
           len_word: int = None,
           agl=None,
           height=2):
	return {"color": color,
	        "bg_color": bg_color,
	        "attrs": attrs,
	        "len_word": len_word,
	        "agl": agl,
	        "height": height}


dDEBUG = {"active": True,
          "title_name": "[DEBUG]",
          "style_text": dstyle(**{"len_word": 25})}
dINFO = {"active": True,
         "title_name": "[INFO]",
         "style_text": dstyle(**{"color": "blue", "len_word": 25})}
dWARNING = {"active": True,
            "title_name": "[WARNING]",
            "style_text": dstyle(**{"color": "yellow", "attrs": ["bold"], "len_word": 31})}
dEXCEPTION = {"active": True,
              "title_name": "[EXCEPTION]",
              "style_text": dstyle(**{"color": "red", "attrs": ["bold"], "len_word": 31})}



__all__ = ["ViewTui", ]

from os import system
from typing import List

import urwid

from console_debugger.helpful.date_obj import ViewRoot
from console_debugger.logic.mg_get_socket import MgGetSocket
from console_debugger.tui.urw_widget import ConsolesColumns
from console_debugger.tui.urw_widget import MenuConsole, ConsoleFrame


class ViewTui(ViewRoot):
	palette = [
		('label_text1', 'light cyan', 'black',),
	]
	urwid.set_encoding('utf8')

	def __init__(self):

		self.console_columns: ConsolesColumns = ConsolesColumns(["1", "2", "3", "4"], self)

		self.menu = MenuConsole(self.ExecuteCommand, root_=self)

		self.gInfoOverlay = urwid.Overlay(self.menu,
		                                  self.console_columns,
		                                  align="center",
		                                  valign="middle",
		                                  width=("relative", 60),
		                                  height=("relative", 60),
		                                  )

		self.loop = urwid.MainLoop(self.console_columns,
		                           palette=ViewTui.palette, )

		self.loop.set_alarm_in(0.1, self.update_widget)

		try:
			super().__init__()
			MgGetSocket.RunThread(self)
			self.loop.run()
		except KeyboardInterrupt:
			print("Exit")
			system("clear")
			exit()

	def update_widget(self, loop=None, data=None):
		"""
		Пока нет обращений к приложению оно не обновляет состояние, поэтому имитируем
		обращение, для обновления данных.
		"""
		self.console_columns.update_widget()
		loop.set_alarm_in(0.1, self.update_widget)

	def ExecuteCommand(self, output_widget: object, command: str):
		"""Глобальная консоль ввода:
- `close` = Скрыть меню
- `help` = Подсказка"""

		command: List[str] = command.split()

		if command[0] == "close":
			self.loop.widget = self.console_columns

		elif command[0] == "help":
			output_widget.set_text(
				f"{ConsolesColumns.keypress.__doc__}\n\n{ViewTui.ExecuteCommand.__doc__}\n\n{ConsoleFrame.ExecuteCommand.__doc__}")

	def PrintInfo(self, text: str):
		self.console_columns.PrintInfo(text)

	def UpdateTitle(self, l_text: List[str]):
		self.console_columns.UpdateTitle(l_text)

	def SendTextInIndex(self, index: int, data: str):
		self.console_columns.SendTextInIndex(index, data)


if __name__ == '__main__':
	...



__all__ = [
	"ConsolesColumns",
	"ConsoleFrame",
	"EditLine",
	"EditListBox",
	"MenuConsole",
]

from typing import List, Callable

import urwid


class ConsolesColumns(urwid.Columns):
	"""
	Колоны с консолями
	"""

	def __init__(self, title_names: List[str], root_: object, focus_column=0, ):
		self.root = root_

		ConsoleFrame.new_max_column(len(title_names))
		widget_list = [ConsoleFrame(name, ) for name in title_names]
		self.__index_column = [focus_column, len(title_names) - 1]  # Для цикличного перемежения
		super().__init__(widget_list, dividechars=0, focus_column=focus_column, min_width=1,
		                 box_columns=None)

	# self.keypress(1,1)

	def UpdateTitle(self, title_names: List[str]):
		"""
		Создать новые консоли
		"""
		for _ in range(self.__index_column[1] + 1):
			self.__del_widget_in_index(0)

		ConsoleFrame.new_max_column(len(title_names))

		for name in title_names:
			self.__append_widget(name)

		self.set_focus(0)
		self.__index_column[0] = 0
		self.__index_column[1] = len(title_names) - 1

	def update_widget(self):
		self.get_focus()

	def SendTextInIndex(self, index_: int, text_: str):
		"""
		Вставить текст в текстовое поле по его индексу:
		"""
		self.contents[index_][0].body_txt.txt.set_edit_pos(0)
		self.contents[index_][0].body_txt.txt.insert_text(text_)
		self.contents[index_][0].body_txt.txt.set_edit_pos(0)

	def PrintInfo(self, text: str):
		self.SendTextInIndex(0, text)

	def keypress(self, size, key):
		"""Горячие клавиши:
- `f1` = Влево
- `f3` = Верх
- `f2` = Вниз
- `f4` = Вправо
- `Tab`= Вправо
- `shif + ЛКМ` = Выделить текст
- `shif + ctrl + C` = Копировать выделенный текст
- `shif + ctrl + V` = Вставить текст"""

		"""
		Обработка перемещений между окнами
		"""
		if key == 'tab':
			self.set_focus(self.__next_focus())
		elif key == 'f1':
			self.set_focus(self.__last_focus())
		elif key == 'f3':
			self.contents[self.focus_col][0].set_focus("body")
		elif key == 'f2':
			self.contents[self.focus_col][0].set_focus("footer")
		elif key == 'f4':
			self.set_focus(self.__next_focus())
		elif key == "f5":
			self.root.loop.widget = self.root.gInfoOverlay

		"""
		Если в фокусе текстовое поле для команд то перенаправляет вывод клавиш в него
		"""
		if self.contents[self.focus_col][0].get_focus() == "footer":
			if key not in ("down", "up"):
				return self.contents[self.focus_col][0].footer.base_widget.keypress([size, ], key)
		else:
			"""		
			Отправляем кнопки в обработчики текстового поля
			"""
			return self.contents[self.focus_col][0].body.keypress(size, key)

	def __del_widget_in_index(self, index):
		self.contents.pop(index)

	def __append_widget(self, title):
		self.contents.append([ConsoleFrame(title), self.options()])

	def __next_focus(self):
		if self.__index_column[0] < self.__index_column[1]:
			self.__index_column[0] += 1
		else:
			self.__index_column[0] = 0
		return self.__index_column[0]

	def __last_focus(self):
		if self.__index_column[0] > 0:
			self.__index_column[0] -= 1
		else:
			self.__index_column[0] = self.__index_column[1]
		return self.__index_column[0]


class ConsoleFrame(urwid.Frame):
	"""
	Объект консоль
		- Заголовок
		- Многострочное текстовое поле
		- Однострочное текстовое поле
	"""

	__max_column: List[int] = [0, 0]

	def __init__(self, title: str, ):

		Ltop_top, Ltop_bot, Rtop_top, Rtop_bot, \
		bod_line, \
		Lbot_top, Lbot_bot, Rbot_top, Rbot_bot, = self.__get_char_table()

		header = urwid.LineBox(urwid.Text(title, align="center"),
		                       tlcorner=Ltop_top,
		                       blcorner=Ltop_bot,
		                       rline=bod_line,
		                       trcorner=Rtop_top,
		                       brcorner=Rtop_bot,
		                       )

		self.body_txt = EditListBox()

		body = urwid.LineBox(self.body_txt,
		                     bline='',
		                     tline='',
		                     rline=bod_line,
		                     )

		footer = urwid.LineBox(EditLine(fun_execute_command=self.ExecuteCommand, output_widget=self.body_txt.txt, ),
		                       tlcorner=Lbot_top,
		                       blcorner=Lbot_bot,
		                       rline=bod_line,
		                       trcorner=Rbot_top,
		                       brcorner=Rbot_bot,
		                       )

		super().__init__(body, header, footer, focus_part='body')

	@staticmethod
	def ExecuteCommand(output_widget: object, command: str):
		"""Локальная консоль ввода:
- `clear` = Отчистить консоль
- `save <name> <path>` = сохранить в файл"""

		command = command.split()

		if command[0] == "clear":
			output_widget.set_edit_text('')

		elif len(command) == 3 and command[0] == "save":

			output_widget.set_edit_pos(0)
			try:
				path_ = "{}/{}".format(command[2].replace("\\", "/"), command[1])
				with open(path_, "w", encoding="utf-8")as f:
					f.write(output_widget.get_edit_text())

				output_widget.text_menu.insert_text(f"# Save {command[1]} >> {path_}\n")

			except (FileNotFoundError, FileExistsError, PermissionError) as e:
				output_widget.insert_text(f"# {e}")

			output_widget.set_edit_pos(0)

	@classmethod
	def new_max_column(cls, i: int):
		cls.__max_column[0] = 0
		cls.__max_column[1] = i

	def __get_char_table(self):
		res = "┌", "├", "┐", "┤", \
		      "│", \
		      '├', "└", "┤", "┘",

		if self.__max_column[1] > 1:
			if self.__max_column[0] == 0:
				res = "┌", "├", "─", "─", \
				      " ", \
				      '├', "└", "─", "─",

			elif self.__max_column[0] + 1 == self.__max_column[1]:
				res = "┬", "┼", "┐", "┤", \
				      "│", \
				      '┼', "┴", "┤", "┘",

			else:
				res = "┬", "┼", "─", "─", \
				      " ", \
				      '┼', "┴", "─", "─",

		self.__max_column[0] += 1

		# print(res)
		# input()

		return res


class EditLine(urwid.Edit):
	"""
	Однострочное поле для ввода текста | команд
	"""

	def __init__(self, output_widget: object, fun_execute_command: Callable[[object, str], None], ):
		self.output_widget: object = output_widget
		self.fun_execute_command = fun_execute_command
		super().__init__(caption="", edit_text="", multiline=False, align="left")

	def keypress(self, size, key):
		"""
		Если нажать`Enter`, то выполнится  self.fun_execute_command

		Если нажать `F5`, то выполнится  self.fun_execute_command с парамером для скрытия окна
		"""
		if key == "enter":
			self.fun_execute_command(self.output_widget, self.get_edit_text())
			self.set_edit_text('')
		elif key == "f5":
			self.fun_execute_command(None, "close")

		super(EditLine, self).keypress(size, key)


class EditListBox(urwid.ListBox):
	"""
	Многострочное текстовое поле
	"""
	__index_style = 0

	def __init__(self):
		self.__index_style = 0
		self.txt = urwid.Edit(multiline=True, align="left")
		body = urwid.SimpleFocusListWalker([self.txt])
		super(EditListBox, self).__init__(body)

	def keypress(self, size, key):
		super(EditListBox, self).keypress(size, key)


class MenuConsole(urwid.LineBox):
	"""
	Меню:
		- Консоль
		- Текстовое поле для вывода
	"""

	def __init__(self, fun_execute_command, root_):
		self.output_menu = urwid.Text("", align="left")
		root_.ExecuteCommand(self.output_menu, "help")

		self.text_menu = EditLine(output_widget=self.output_menu, fun_execute_command=fun_execute_command, )

		line_menu = urwid.Divider("-")
		body = urwid.SimpleFocusListWalker([self.text_menu, line_menu, self.output_menu])
		self.list_box_menu = urwid.ListBox(body)
		super().__init__(self.list_box_menu, title="",
		                 title_align="center",
		                 title_attr=None,
		                 tlcorner=u'┌',
		                 tline=u'─',
		                 lline=u'│',
		                 trcorner=u'┐',
		                 blcorner=u'└',
		                 rline=u'│',
		                 bline=u'─',
		                 brcorner=u'┘')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['console_debugger',
 'console_debugger.gui',
 'console_debugger.helpful',
 'console_debugger.logic',
 'console_debugger.test',
 'console_debugger.tui']

package_data = \
{'': ['*'], 'console_debugger.gui': ['static/icons.ico']}

install_requires = \
['urwid>=2.1.2,<3.0.0']

setup_kwargs = {
    'name': 'console-debugger',
    'version': '0.1.8.1',
    'description': 'Это подключаемая библиотека, для удобного вывода отладочной информации.',
    'long_description': '# Что это?\n\nЭто подключаемая библиотека, для удобного вывода\nотладочной информации.\n\n---\n\n# Как использовать ?\n\n## 1 Сначала нужно создать экземпляры класса Debugger\n\nМожно вручную указать параметры экземпляра\n\n- `Debug_Name = Debugger(active: bool,,title_name: str, consoleOutput: bool = True, fileConfig: Optional[Dict] = None style_text: Optional[dstyle] = None)`\n\n     - `active` = on/off жизни экземпляра, Если `False` экземпляр \n    будет игнорировать вызов, а также будет добавлен в массив `Debugger.AllCountSleepInstance`.  \n       \n    - `title_name` = Уникальное имя экземпляра которое будет отображаться в выводе.  \n    \n    - `consoleOutput` = on/off отображения в консоль или другие\n    визуальные выходы, **не влияет на запись в файл!**.  \n    \n    - `fileConfig` = Конфигурация записи в файл, входные параметры такие же как и у стандартной функции `open()`  \n        передавать в формате `Dict{"file":"test.log", ... }`. Для удобного формирования параметров можно  \n         пользоваться функцией `dopen()`.  \n    \n    - `style_text` = Стиль отображения текста. Для удобного формирования параметров можно  \n        пользоваться функцией `dstyle`.  \n\n\nИли использовать готовые параметры\n\n- `Debug_Name = Debugger(**dDEBUG)`\n    - dDEBUG\n    - dINFO\n    - dWARNING\n    - dEXCEPTION\n\n## 2 Установить глобальный стиль для всех экземпляров\n\nЭта команда влияет на все экземпляры `Debugger`\n\n- `Debugger.GlobalManager(global_status=None, typePrint: Optional[str] = "grid"):`\n    - `global_status` = Вы можете on/off все экземпляры разом\n    - `typePrint=` = Глобальный стиль отображения данных\n        - `"grid"` = Стиль таблица \n        \n        ![](https://i.imgur.com/Kif40aB.png)\n        \n        - `"socket"` = Данные будут отправляться по сокету, в данный момент есть два варианта прослушивания сокета.\n        Через GUI [Tkinter](#про-режим-отображения-tkinter) `console_debugger/main.py gui` или через TUI [Urwid](#про-режим-отображения-urwid) \n        `console_debugger/main.py tui`.\n        Если в процессе отправки данных через сокет возникнут ошибки, данные будут сохранены в файл.\n             \n        - `None` = Без стиля \n        \n        ![](https://i.imgur.com/byg84id.png)\n\n## 3 Использовать в коде\n\nИспользовать стандартную функцию `print`.\n- `print(text, file= Debug_Name)`\n    - `text` = Строка\n    - `Debug_Name` = Имя экземпляра `Debugger`\n\nИспользовать функцию `printD`. Преимущество в том что может принимать несколько переменных и соеденять их.\n\n- `printD(Debug_Name, text, *args, sep=\' \', end=\'\\n\')`\n    - `Debug_Name` = Имя экземпляра `Debugger`\n    - `text` = Строка\n    - `*args` = Данные преобразуются в тип `str`\n    - `sep=\' \', end=\'\\n\'` = такие же, как и у встроенной функции `print()`\n\n---\n\n# Примеры\n\n## Использовать свои стили, вызывать экземпляры напрямую\n\n\nДля наглядности создадим функцию для генерации случайного слово\n```python\nimport random\nimport string             \n# Сгенерировать случайное слово\nrandom_word = lambda: "".join(random.choice(string.ascii_letters) for j in range(random.randint(1, 40)))\n```\n\n```python\nfrom console_debugger import *\n\nDebug = Debugger(True,title_name="[DEBUG]",\n\n                 fileConfig=dopen(file="debug.log",\n                                  mode="a",\n                                  encoding="utf-8"),\n\n                 style_text=dstyle(bg_color="bg_blue",\n                                   len_word=21)\n                 )\nInfo = Debugger(True,title_name="[INFO]",\n\n                fileConfig={"file": "info.log",\n                            "mode": "a", "encoding": "utf-8"},\n                style_text=dstyle(len_word=25),\n\n                consoleOutput=False\n                )\nWarning = Debugger(True,"[WARNING]", style_text=dstyle(len_word=25))\n\nDebugger.GlobalManager(typePrint="grid")\n\nif __name__ == \'__main__\':\n    for i in range(10):\n        printD(Debug, random_word())\n        printD(Warning, random_word())\n        printD(Info, random_word())\n```\n\n## Использовать готовые стили, вызывать `printD`\n\n```python\nfrom console_debugger import *\n\nDebug = Debugger(**dDEBUG)\nInfo = Debugger(**dINFO)\nWarning = Debugger(**dWARNING)\n\nDebugger.GlobalManager(typePrint="grid")\n\nif __name__ == \'__main__\':\n    for i in range(10):\n        printD(Debug, random_word())\n        printD(Warning, random_word())\n        printD(Info, random_word())\n```\n\n## Использовать готовые стили, вызывать `print`\n```python\nfrom console_debugger import *\n\nDebug = Debugger(**dDEBUG)\nInfo = Debugger(**dINFO)\nWarning = Debugger(**dWARNING)\n\nDebugger.GlobalManager(typePrint="socket")\n\nif __name__ == \'__main__\':\n    for i in range(10):\n\tprint(random_word(), file=Debug)\n\tprint(random_word(), file=Info)\n\tprint(random_word(), file=Warning)\n```\n\n## Использовать `soket`\n```python\nfrom console_debugger import *\n\n\nDebug = Debugger(**dDEBUG)\nInfo = Debugger(**dINFO)\nWarning = Debugger(**dWARNING)\nTEST = Debugger(True,"TEST")\n\nDebugger.GlobalManager(typePrint="socket")\n\n\nfor i in range(10):\n    printD(Debug, random_word())\n    printD(Info, random_word())\n    printD(Warning, random_word())\n    printD(TEST, random_word())\n```\n\n## Использование Глобального режима on/off \nСоздать два режима запуска Debug/Release\n![](https://i.imgur.com/guFWf3O.png)\n\n`main.pyw`\n```python\nimport sys\nfrom console_debugger import Debugger\nfrom app.viwe import Windows # При импорте должны быть созданы все экземпляры\n\nif __name__ == \'__main__\':\n    for param in set(sys.argv):\n        if param == "--d":\n            Debugger.GlobalManager(typePrint="grid") # Задать глобальный стиль всем экземпляром\n            break\n    else:\n        Debugger.GlobalManager(global_status=False) # Если нет параметров отключаем все экземпляры \n    Windows()\n```\nВ других модулях создаем необходимые экземпляры\n```python\nfrom console_debugger import *\nHotKeyD = Debugger(True, "[HotKey]")\nPressKeyD = Debugger(True, "[PressKey]")\nInfoD = Debugger(**dINFO)\nResD = Debugger(True, "[Result]", style_text=dstyle(len_word=25, height=4))\n\nprintD(HotKeyD,"Crtl+c")\n```\n\n# Пример трассировки переменных\n\n```python\nimport random\nimport string\n\nrandom_word = lambda: "".join(random.choice(string.ascii_letters) for j in range(random.randint(1, 40)))\n\nfrom console_debugger import *\n\nif __name__ == \'__main__\':\n\ta = Debugger(**dDEBUG)\n\tb = Debugger(**dINFO)\n\tc = Debugger(**dEXCEPTION)\n\n\tDebugger.GlobalManager(typePrint="socket")\n\n\tTracingName1 = "1"\n\tTracingName2 = ["1"]\n\tTracingName3 = "1",\n\n\tfor x in range(10):\n\t\tprintD(a, TracingName1)\n\t\tprintD(b, TracingName2)\n\t\tprintD(c, TracingName3)\n```\n![](https://i.imgur.com/8ctYa9G.png)\n\n\n## Использование во `Flask`\n\nПоместить `Debugger.GlobalManager` в `@app.before_first_request`\n```python\nfrom console_debugger import *\nfrom flask import * \n\nSECRET_KEY = "123_very_hard_password"\napp = Flask(__name__)\n\n # Экземпляры в глобальной области видимости\ncookDeb = Debugger(True, "[Cook]")\nsessionDeb = Debugger(True, "[Session]")\n\n@app.before_first_request\ndef deb():\n    # Tkinter будет перезапускаться при каждом обновление сервера\n    Debugger.GlobalManager(typePrint="socket")\n\n@app.route("/login", methods=[\'POST\', \'GET\'])\ndef login():\n    global data, cookDeb, sessionDeb\n    cook = "no"\n    ses = "no"\n\n    # Получить куки если есть\n    if request.cookies.get("logged"):\n        cook = request.cookies.get("logged")\n\n    # Получить данные из сессии если есть\n    if "SessioN" in session:\n        ses = session.get("SessioN")\n\n    printD(cookDeb, cook)\n    printD(sessionDeb, ses)\n\n    res = make_response(render_template("login.html", cook=cook, session=ses))\n    res.set_cookie(key="logged", value="yes", max_age=3)\n    session["SessioN"] = "yes"\n    return res\n\nif __name__ == \'__main__\':\n    app.run(debug=True)\n```\n`"login.html"`\n```html\n<script>\n    document.cookie = "ex=1;";\n    if (!document.cookie) {\n        alert("Этот сайт требует включение cookie");\n    }\n</script>\nCook: {{ cook }}\n<p></p>\nSession: {{ session }}\n<form action="/login" method="post" class="form-contact">\n    <p><label>Name </label><input type="text" name="username" value="" required/>\n    <p><label>Passwortitled </label><input type="text" name="password" value="" required/>\n    <p><input type="submit" value="Send"/>\n</form>\n```\n\n## Использование в `Django`\n```python\nimport os\nfrom console_debugger import Debugger\n\n\nif not os.environ.get(\'console_debugger\', False):\n    os.environ[\'console_debugger\'] = "True"\n    print("1")\nelse:\n    print("2")\n    Debugger.GlobalManager(typePrint="socket")\n```\n# Советы\n\n\n## Вы можете использовать шаблон для комментирования `printD`\nТак как входные параметры функции являются не изменяемыми обметками, \nто они копируются в функцию `printD`, чтобы н тратить лишении наносекунды\nна эту операцию, вы можете закомментировать все вызовы этой функции.\n \n ```cmd\n\\s{4}printD+ \n#none: printD\n```\n \n![Замена](https://i.imgur.com/cmzsU24.png)\n\nА потом раскомментировать\n\n ```cmd\n#none: printD \n    printD\n```\n![Замена](https://i.imgur.com/P0nugCD.png)\n\n \n\n## Про режим отображения Tkinter\n\n![](https://i.imgur.com/YJFYv57.png)\n- Если нажать на заголовок консоли, то они выполнят команду из нижней консоли\n- Если нажать нижнею кнопку `save geometry` то вы\nсохраните положение окна для следующих запусках, размеры сохранять в  `console_debugger/gui/static/config.txt`\n- Если закрыть окно `Tkinter`, до завершения главного потока, то данные будут отправляться в консоль `typePrint=None"`\n- Можно сохранить весь текст из консоли в файл. Для этого нужно ввести в нижнею консоль `save <NAME_FILE> <PATH>` и нажать `Enter`\n- Можно получить глобальную информацию о сокете написав в любую нижнею консоль `g info` и нажать `Enter`\n- Можно отчистить консоль вывода если написать в нижнею консоль `clear` и нажать `Enter`\n\n### Установка Tkinter на Linux\nArchLinux\n```bush\nsudo pacman -S tk\n\nxrdb -load /dev/null  \nxrdb -query \n```\n\n## Про режим отображения Urwid\n\n![](https://i.imgur.com/EQ8MLhG.png)\nРаботает также в терминале Pycharm\n![](https://i.imgur.com/hJlPVZn.png)\n\n\nЭтот режим поддерживается в терминалах Linux. Программа находиться в `console_debugger/tui/main.py`.\n\n- Можно сохранить весь текст из консоли в файл. Для этого нужно ввести `save <NAME_FILE> <PATH>` \n- Горячие клавиши\n    - `f1` = Влево\n    - `f3` = Верх\n    - `f2` = Вниз\n    - `f4` = Вправо\n    - `Tab`= Вправо\n    - `shif + ЛКМ` = Выделить текст\n    - `shif + ctrl + C` = Копировать выделенный текст\n    - `shif + ctrl + V` = Вставить текст\n\n\n  \n- Можно отчистить консоль вывода если написать в нижнею консоль `clear` и нажать `Enter`\n\n## Про доступную информации об экземпляре `Debugger`\n- public set:\n    + `consoleOutput` = Переключить on/off вывод в консоль\n    + `style_text` = Задать другой стиль текста\n    + `active()` = Включить дебагер\n    + `deactivate()` = Отключить дебагер\n    \n- public get:\n    + `title_name` = Уникальное имя дебагера\n    + `fileConfig` = Конфигурация для файла\n    + `AllCountActiveInstance()` = Все активные дебагеры\n    + `AllCountSleepInstance()` = Все приостановленный дебагиры\n    + `AllUseFileName()` = Все используемые имена файлов\n    + `AllInstance()` = Все экземпляры дебагеров\n\n\n',
    'author': 'Denis Kustov',
    'author_email': 'denis-kustov@rambler.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/denisxab/console_debugger',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



from sys import argv, path
from helpful.template_obj import add_sys_path_if_not, rel_path
add_sys_path_if_not(rel_path(-2), path)


from console_debugger.gui.tk_terminal import ViewGui
from console_debugger.tui.urwid_terminal import ViewTui

if __name__ == '__main__':
	for param in set(argv):
		if param == "tui":
			ViewTui()
			break
		elif param == "gui":
			ViewGui()
			break
	else:
		print("tui OR gui")
